/*
 * Copyright (c) 2006, Adam Dunkels
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

	#ifndef __TOKENIZER_H__
		#define __TOKENIZER_H__


			#ifndef BASIC_CONFIG_FILE
				#define BASIC_CONFIG_FILE "basic_cfg.h"
			#endif


			#define TOKENIZER_CONFIG
				#include BASIC_CONFIG_FILE
			#undef TOKENIZER_CONFIG


			typedef struct
			{
				char type;
				union {
					char text [MAX_USER_KEYWORD_LEN];
					int  value;
				};
			} T_USER_FUNC_DATA;


			typedef T_USER_FUNC_DATA T_USER_FUNC_DATAS [MAX_USER_PARAMS];

			typedef void (* T_BASIC_FUNCTION)  (void);
			typedef int  (* T_BASIC_EXPR_FUNC) (int a, int b);
			
			typedef int  (* T_BASIC_USER_FUNC) (T_USER_FUNC_DATAS stDatas);
			typedef int  (* T_BASIC_USER_ARRF) (int index);


			#define BASIC_TOKENS

				enum
				{
					TOKENIZER_ERROR,
					TOKENIZER_ENDOFINPUT,
					TOKENIZER_NUMBER,
					TOKENIZER_STRING,
					TOKENIZER_VARIABLE,
					TOKENIZER_NAME,

					TOKENIZER_KEYWORDS = TOKENIZER_NAME,

					#define TOKEN_WORD(name,text,func) name,
					#define TOKEN_CHAR(name,char,func)
						#include BASIC_CONFIG_FILE
					#undef TOKEN_WORD
					#undef TOKEN_CHAR

					TOKENIZER_KEYCHARS, 
					TOKENIZER_KEYCHARS_1 = TOKENIZER_KEYCHARS - 1,

					#define TOKEN_WORD(name,text,func) 
					#define TOKEN_CHAR(name,char,func) name,
						#include BASIC_CONFIG_FILE
					#undef TOKEN_WORD
					#undef TOKEN_CHAR


					TOKENIZER_NUM_TOKENS
				};

				#define NO_FUNCTION		dummyfunc
				#define NO_EXPR_FUNC	dummyexprfunc

				#define TOKEN_WORD(name,text,func) extern void func (void);
				#define TOKEN_CHAR(name,char,func) extern int  func (int a, int b);
					#include BASIC_CONFIG_FILE
				#undef TOKEN_WORD
				#undef TOKEN_CHAR

			#undef BASIC_TOKENS


			#define BASIC_USER_EXTENSIONS
				enum {
					#define USER_FUNCTION(name,text,func,param)		name, 
					#define USER_VARIABLE(name,text,var)					name,
					#define USER_ARRAY(name,text,var,size)				name, 
						#include BASIC_CONFIG_FILE
					#undef USER_FUNCTION
					#undef USER_VARIABLE
					#undef USER_ARRAY
					BASIC_NUM_USER_EXTENSIONS
				};

				#define USER_FUNCTION(name,text,func,param)			extern int func (T_USER_FUNC_DATAS stDatas);
				#define USER_VARIABLE(name,text,var)						extern int var;
				#define USER_ARRAY(name,text,var,size)					extern int var [size];
					#include BASIC_CONFIG_FILE
				#undef USER_FUNCTION
				#undef USER_VARIABLE
				#undef USER_ARRAY

			#undef BASIC_USER_EXTENSIONS


			void	tokenizer_init					(const char *program);
			void	tokenizer_next					(void);
			int		tokenizer_token					(void);
			int		tokenizer_num						(void);
			int		tokenizer_variable_num	(void);
			void	tokenizer_string				(char *dest, int len);
			int		tokenizer_userdata			(int *type, int *data, void **ptr);

			int		tokenizer_finished			(void);
			void	tokenizer_error_print		(void);

			void  execbasicfunc						(void);
			int		execbasicexprfunc				(int op, int a, int b);

		#endif /* __TOKENIZER_H__ */
